/****************************************************************
Copyright (c) 1998 Sandro Sigala <ssigala@globalnet.it>.
All rights reserved.

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name of the author not be used in
advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

The author disclaim all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall the author be liable for any
special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
****************************************************************/

#ifndef GAMEWIDGET_H
#define GAMEWIDGET_H

#include <qwidget.h>
#include <qlist.h>

#include "kzacman.h"

class QTimer;
class KAudio;

enum Direction { North, South, East, West };

struct Monster {
	int xpos, ypos;
	Sprite sprite;
	Direction dir;
	bool saw;

	Monster(int x, int y, Sprite s)
		: xpos(x), ypos(y), sprite(s), dir(North), saw(false) {}
};

class GameWidget : public QWidget {
	Q_OBJECT

signals:
	void changedStats(int, int, int);
	void noStats();

public:
	bool in_game, in_pause;
	bool do_sounds;

	GameWidget(QWidget *parent=0, const char *name=0);
	~GameWidget();

private:
	KAudio *audio;
	int xpos, ypos;
	QPixmap *sprites;	// The sprites array
	Sprite *map;		// The map array
	int map_width, map_height;
	Sprite sprite_wall;	// The sprite used for walls
	Sprite sprite_back;	// The sprite used for non-walls
	Direction dir;		// The orientation of Zacman
	Skill game_skill;
	QList<Monster> monsters;
	int timerSecs, timerMonsters;
	int num_points;
	int num_secs;
	int num_potion_secs;
	bool show_balloon;
	Sprite balloon;

	Sprite& ref(int x, int y) {
		return map[y*map_width + x];
	}

	Sprite xy(int x, int y) {
		if (x < 0 || y < 0 || x >= map_width || y >= map_height)
			return sprite_wall;
		return map[y*map_width + x];
	}

	void playSound(Sound s);
	void loadSprites();
	void loadSprite(Sprite spr, const char *path);
	void loadMaskedSprite(Sprite spr, char *path1, const char *path2);
	void deleteMonsters();
	Monster *findMonster(int x, int y);
	void addMonster(int x, int y);
	void killMonster(Monster *mp);
	void updateMonsters();
	bool lookMonster(Monster *m);
	Direction monsterFindDirection(Monster *m, Direction exclude);
	void createMap(int x, int y);
	void checkPosition(int x, int y);
	void showBalloon(Sprite s, int secs);
	void die();

public slots:
	void newGame(Skill skill);
	void goUp();
	void goDown();
	void goLeft();
	void goRight();

protected:
	void timerEvent(QTimerEvent *e);
	void paintEvent(QPaintEvent *);
};

#endif // !GAMEWIDGET_H
